import java.util.Random;

public class PoissonSender implements Event, ApplicationSender {
    double lambda;
    TransportSender transport;
    int count;
    int max_count;
    Random rnd;
    boolean blocked;
    byte [] pending_data;

    public int DATA_SIZE = 1500;

    public PoissonSender(int c, double l) {
	count = 1;
	max_count = c;
	lambda = l;
	rnd = new Random();
	blocked = false;
	pending_data = null;
    }

    public void start(double time) {
	Simulation.schedule_event(time, this);
    }	

    public void connect(TransportSender t) {
	transport = t;
    }

    private double exp_random(double lambda) {
	return  Math.log(1 - rnd.nextDouble())/(-lambda);
    }

    static byte [] new_packet(int seq_number) {
	return ("seq:" + seq_number).getBytes();
    }

    public void stop_sender() {
	blocked = true;
    }

    public void continue_sender() {
	blocked = false;
	if (pending_data != null)
	    process();
    }

    public void process() {	// action for "wake up" event
	if (count <= max_count) {
	    byte [] data;
	    if (pending_data != null)
		data = pending_data;
	    else
		data = new_packet(count);

	    System.out.printf("%f ", Simulation.time());
	    System.out.println("Sender:" + count
			       + ": data=" + new String(data));
	    if (!transport.send(data, DATA_SIZE)) {
		pending_data = data;
		System.out.printf("%f ", Simulation.time());
		System.out.println("Sender:" + count + ": rejected");
		return;
	    } 
	    Simulation.schedule_event(exp_random(lambda), this);
	    count += 1;
	    pending_data = null;
	} else {
	    System.out.printf("%f ", Simulation.time());
	    System.out.println("Sender: terminated.");
	}
    }
}
