import java.util.*;

public class TCPSimulation {
    public static void main(String [] args) {
	// default parameters
	double lambda = 1.0;	// 1 packet per second
	double rate = 1000000; 	// 1Mbytes/second
	double delay = 0.1; 	// 100 milliseconds
	double error_probability = 0.0;
	int n = 100;		// 100 packets
	int qsize = 100; 	// TX queue size for link

	try {
	    for(int i = 0; i < args.length; ++i) {
		if (args[i].startsWith("lambda=")) 
		    lambda = Double.parseDouble(args[i].substring(7));
		else if (args[i].startsWith("rate=")) 
		    rate = Double.parseDouble(args[i].substring(5));
		else if (args[i].startsWith("latency=")) 
		    delay = Double.parseDouble(args[i].substring(8));
		else if (args[i].startsWith("delay=")) 
		    delay = Double.parseDouble(args[i].substring(6));
		else if (args[i].startsWith("error=")) 
		    error_probability = Double.parseDouble(args[i].substring(6));
		else if (args[i].startsWith("qsize=")) 
		    qsize = Integer.parseInt(args[i].substring(6));
		else if (args[i].startsWith("n=")) 
		    n = Integer.parseInt(args[i].substring(2));
		else
		    throw new Exception("unknown option: " + args[i]);
	    }
	} catch (Exception ex) {
	    System.err.println(ex);
	    System.err.println("usage: Link [options...]\n\tlambda=<double> sender app output rate packets per second (default 1.0/s)\n\trate=<double>   link rate in bytes per second (default 1MB/s)\n\tdelay=<double>  link delay in seconds (default 0.1s)\n\terror=<double>  link error probability (default 0.0)\n\tn=<int>         number of packets sent by sender app (default 100)\n\tqsize=<int>     queue size in network link (default 100)");
	    System.exit(1);
	}

	PoissonSender sender_app1 = new PoissonSender(n, lambda);
	PoissonSender sender_app2 = new PoissonSender(n, lambda);
	LoggerReceiver receiver_app1 = new LoggerReceiver();
	LoggerReceiver receiver_app2 = new LoggerReceiver();

	TCPSender sender_transport1 = new TCPSender(1111, 2222); 
	TCPSender sender_transport2 = new TCPSender(1112, 2223); 

	TCPReceiverStack receiver_stack = new TCPReceiverStack(); 

	TCPReceiver receiver_transport1 = new TCPReceiver(2222, 1111); 
	TCPReceiver receiver_transport2 = new TCPReceiver(2223, 1112); 

	BasicLink link = new BasicLink(qsize);

	sender_app1.connect(sender_transport1);
	sender_app2.connect(sender_transport2);

	sender_transport1.connect(link);
	sender_transport2.connect(link);

	link.connect(receiver_stack);

	receiver_stack.connect(receiver_transport1, receiver_app1);
	receiver_stack.connect(receiver_transport2, receiver_app2);

	link.set_rate(rate);
	link.set_delay(delay);
	link.set_error_probability(error_probability);

	sender_app1.start(0.0);
	sender_app2.start(100.1);

	Simulation.run();
    }
}
